package org.apache.bcel.classfile;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public final class Unknown extends Attribute {
   private static final Unknown[] EMPTY_ARRAY = new Unknown[0];
   private static final Map<String, Unknown> UNKNOWN_ATTRIBUTES = new HashMap();
   private byte[] bytes;
   private final String name;

   static Unknown[] getUnknownAttributes() {
      Unknown[] var0;
      try {
         var0 = (Unknown[])UNKNOWN_ATTRIBUTES.values().toArray(EMPTY_ARRAY);
      } finally {
         UNKNOWN_ATTRIBUTES.clear();
      }

      return var0;
   }

   public Unknown(int nameIndex, int length, byte[] bytes, ConstantPool constantPool) {
      super((byte)-1, nameIndex, length, constantPool);
      this.bytes = bytes;
      this.name = constantPool.getConstantUtf8(nameIndex).getBytes();
      UNKNOWN_ATTRIBUTES.put(this.name, this);
   }

   Unknown(int nameIndex, int length, DataInput input, ConstantPool constantPool) throws IOException {
      this(nameIndex, length, (byte[])null, constantPool);
      if (length > 0) {
         this.bytes = new byte[length];
         input.readFully(this.bytes);
      }

   }

   public Unknown(Unknown c) {
      this(c.getNameIndex(), c.getLength(), c.getBytes(), c.getConstantPool());
   }

   public void accept(Visitor v) {
      v.visitUnknown(this);
   }

   public Attribute copy(ConstantPool constantPool) {
      Unknown c = (Unknown)this.clone();
      if (this.bytes != null) {
         c.bytes = (byte[])this.bytes.clone();
      }

      c.setConstantPool(constantPool);
      return c;
   }

   public void dump(DataOutputStream file) throws IOException {
      super.dump(file);
      if (super.getLength() > 0) {
         file.write(this.bytes, 0, super.getLength());
      }

   }

   public byte[] getBytes() {
      return this.bytes;
   }

   public String getName() {
      return this.name;
   }

   public void setBytes(byte[] bytes) {
      this.bytes = bytes;
   }

   public String toString() {
      if (super.getLength() != 0 && this.bytes != null) {
         int limit = true;
         String hex;
         if (super.getLength() > 10) {
            byte[] tmp = Arrays.copyOf(this.bytes, 10);
            hex = Utility.toHexString(tmp) + "... (truncated)";
         } else {
            hex = Utility.toHexString(this.bytes);
         }

         return "(Unknown attribute " + this.name + ": " + hex + ")";
      } else {
         return "(Unknown attribute " + this.name + ")";
      }
   }
}
